//////////////////////////////////////////////////////////////////
// Battleship                                                   //                                                         //
// Term Project: Comp 2535 - Visual Tools                       //
// BCIT ~ 2001                                                  //
//                                                                               //
//   Young Cha                                                  //
//   Toby Lang                                                  //
//   Brock Haywood                                              //
//   Randy Lukashuk                                             //
//                                                              //
//////////////////////////////////////////////////////////////////

//Playsound('sounds/SubSunk.wav',0,SND_ASYNC);

//////////////////////////////////////////////////////////////////
// Internet Add-On : Randy                                      //
//                                                              //
// ADDED FUNCTIONS                                              //
//      INTERNET FUNCTIONS                                      //
//      ENCODE / DECODE FUNCTIONS                               //
//
// CHANGED FUNCTIONS                                            //
//
//                                                              //
// OTHER CHANGES / NOTES                                        //
//      - Wrote Internet.pas
//      - Wrote Chat.pas
//      - Deleted ShowCoord Function                            //
//      - Added GLOBAL -> InternetTurn : Integer;
//      - Changed BoatPlaced() function
//      - Set events on all the startgame functions
//      - Changed almost all placement functions
//      - Changed PaintGame() function
//      - Changed CheckBoard()
//      - Addded DeclareLoss()
//      - Changed BoardMouseDown()
//
//////////////////////////////////////////////////////////////////
unit Battleship;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, Menus, ExtCtrls, ComCtrls, NewGame, OleCtrls,
  MSWinsockLib_TLB,MMSystem,Math, MPlayer, jpeg;

const
  NItems = 10; //# of cells on board

  //Filenames for the game pics
  NS_ACARRIER_PIC = 'Media\NS_Pictures\NS_Aircraft.bmp';
  NS_SUBMARINE_PIC = 'Media\NS_Pictures\NS_Sub.bmp';
  NS_CARRIER_PIC = 'Media\NS_Pictures\NS_Carrier.bmp';
  NS_BATTLESHIP_PIC = 'Media\NS_Pictures\NS_Battleship.bmp';
  NS_DESTROYER_PIC = 'Media\NS_Pictures\NS_Destroyer.bmp';
  WE_ACARRIER_PIC = 'Media\WE_Pictures\WE_Aircraft.bmp';
  WE_SUBMARINE_PIC = 'Media\WE_Pictures\WE_Sub.bmp';
  WE_CARRIER_PIC = 'Media\WE_Pictures\WE_Carrier.bmp';
  WE_BATTLESHIP_PIC = 'Media\WE_Pictures\WE_Battleship.bmp';
  WE_DESTROYER_PIC = 'Media\WE_Pictures\WE_Destroyer.bmp';
  HIT_PIC = 'Media\Hit1.bmp';
  MISS_PIC = 'Media\Miss1.bmp';
  PLACEMENT_PIC = 'Media\Position.bmp';

  //Filenames for the game sounds
  SUNK_BOAT_WAV = 'Media\sounds\BoatSunk.wav';
  SUNK_BATTLESHIP_WAV = 'Media\sounds\BattleShipSunk.wav';
  START_WAV = 'Media\sounds\GameBoardSet.wav';
  HIT_WAV = 'Media\sounds\Hit.wav';
  MISS_WAV = 'Media\sounds\Miss.wav';
  FIRE_WAV = 'Media\sounds\Missile.wav';
  WIN_WAV = 'Media\sounds\GameBoardSet.wav';
  LOSE_WAV = 'Media\sounds\SubSunk.wav';

type
  TGameMode = (gmNone, gmSinglePlayer, gmMultiplayer, gmInternet);
  TShip = (sNone,sBattleship, sACarrier, sCarrier, sSubmarine, sDestroyer);
  TCellState = (csDefault, csHit, csMiss, csPlacement);
  TDirection = (dHorizontal, dVertical, dNull);

  TShipLoc = record
    StartPosX : Integer;
    StartPosY : Integer;
    EndPosX : Integer;
    EndPosY : Integer;
  end;

  //highscore type
  Thighscore = record
          name : string;
          score : integer;
          date : string;
  end;

  TBattleCell = record
    Ship: TShip; //type of ship...
    State: TCellState; //hit or miss or default...
    ShipLoc : TShipLoc; //start and end pos...
  end;

  //player board
  TBattleBoard = array[1..2,0..NItems-1,0..NItems-1] of TBattleCell;
  TBearing = (bNorth, bWest, bSouth,bEast);

  TPoint = record
   X,Y:Integer;
  end;

  TBoatFlags = record
    ACarrierSunk: Boolean;
    BattleShipSunk: Boolean;
    CarrierSunk: Boolean;
    SubmarineSunk: Boolean;
    DestroyerSunk: Boolean;
  end;

  TGame = record
    BoardArray : array[1..2,0..(nitems-1),0..(nitems-1),0..5] of Byte;
    Play: Boolean;        //play mode
    PlayerName : array[1..2] of String[20]; //player names
    CurrentPlayer : Integer; //current player number
    OpposingPlayer : Integer; //opposing player number
    PlayerFlagArray : array[1..2,0..4] of Boolean;
    GameModeByte : Byte;
    Player1Ready : Boolean; // player1 placed?
    Player2Ready : Boolean; // player2 placed?
    ShowAll : Boolean;
    HideBoats : Boolean;
    FreezeGame : Boolean;
   end; //End TSaveGame

  TSavedGame = record
    SavedGame : TGame;
  end;

  TFrmBattleShip = class(TForm)
    Board1: TDrawGrid;
    Board2: TDrawGrid;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    NewGame1: TMenuItem;
    TopScores1: TMenuItem;
    N1: TMenuItem;
    Quit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    btnDonePlacing: TButton;
    btnPlay: TButton;
    gbPlace: TGroupBox;
    btnACarrier: TButton;
    btnBattleShip: TButton;
    btnCarrier: TButton;
    btnSubmarine: TButton;
    btnDestroyer: TButton;
    lblPlayer1: TLabel;
    lblPlayer2: TLabel;
    Shape1: TShape;
    lblTitle: TLabel;
    StatusBar1: TStatusBar;
    LoadGame1: TMenuItem;
    SaveGame1: TMenuItem;
    N2: TMenuItem;
    btnRandomPlace: TButton;
    btnChat: TButton;
    TCPServer: TWinsock;
    TCPClient: TWinsock;
    ShapeP2: TShape;
    ShapeP1: TShape;
    Shape2: TShape;
    MediaPlayer: TMediaPlayer;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;

    //GAME AND FORM EVENTS
    procedure Quit1Click(Sender: TObject);
    procedure NewGame1Click(Sender: TObject); // Menu item to start a new game
    procedure About1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject); // Resets all the ship select buttons
    procedure btnDonePlacingClick(Sender: TObject);
    procedure BoardMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnPlaceShipClick(Sender: TObject);
    procedure btnPlayClick(Sender: TObject);
    procedure BoardSelectCell(Sender: TObject; ACol, ARow: Integer;
      var CanSelect: Boolean);
    procedure FormPaint(Sender: TObject);
    procedure btnRandomPlaceClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure MediaPlayerNotify(Sender: TObject);
    
    //PLACMENT FUNCTIONS
    procedure PlaceBoat(WhichMap, Col, Row: Integer);
    procedure BoatPlaced(WhichMap,StartPtX,StartPtY,EndPtX,EndPtY: Integer; Ship:TShip);
    procedure PlaceInProgress(WhichMap, Col, Row, Size: Integer);
    procedure SetEndPt(WhichMap, Col, Row: Integer; Ship:TShip);
    procedure PlaceHorizontal(WhichMap, A, B, C: Integer; Ship:TShip);
    procedure PlaceVertical(WhichMap, A, B, C: Integer; Ship:TShip);
    procedure SuccessPlace(Ship:TShip);
    procedure CheckVertical(WhichMap, Col, Row1, Row2: Integer);
    procedure CheckHorizontal(WhichMap, Row, Col1, Col2: Integer);
    procedure ShowButtons();
    function CheckStatus(): Boolean;
    procedure SetPlacements(WhichMap, Col, Row, Size: Integer);
    procedure ClearPlacements(WhichMap : Integer);

    //PAINT FUNCTIONS
    procedure PaintGame();
    procedure PaintCellStates(WhichMap : Integer);
    procedure PaintShips(WhichMap : Integer);
    procedure PaintShip(WhichMap,Col,Row : Integer);
    procedure ErasePaint(WhichMap : Integer);
    function GetBearing(Stx,Sty,Endx,Endy : Integer):TBearing;
    procedure ColorCell(WhichMap, Col, Row: Integer; color: TColor);
    procedure PaintPic(WhichMap : Integer; Coords : TRect; Pic : TBitmap);

    //OTHER GAME FUNCTIONS
    function SizeSelect(): Integer;
    procedure CannonFired(WhichMap, Col, Row: Integer);
    procedure CheckBoard(WhichMap:Integer);
    function CheckSink(WhichMap:Integer;WhichShip:TShip):Boolean;
    procedure EndTurn();
    procedure CurrentMouseToCell(X,Y:Integer; var ACol,ARow : Integer);
    procedure OpposingMouseToCell(X,Y:Integer; var ACol,ARow : Integer);
    function GetShipName(ShipType : TShip) : String;
    procedure StartNewGame();
    procedure StartSinglePlayerGame();
    procedure StartMultiPlayerGame();
    procedure StartInternetGame();
    procedure ShowGame();
    procedure HideGame();
    procedure ResetGame();
    procedure DeclareVictory(WhichPlayer : Integer);
    procedure RandomPlace(WhichMap:Integer);

    //AI FUNCTIONS
    function GetShotCoord() : TPoint;
    procedure DoComputerTurn();
    procedure Dummy(Sender: TObject;
        Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    //INTERNET FUNCTIONS
    procedure DecodeData(Data:OLEVariant);
    procedure SendData(const DataString: String);
    procedure btnChatClick(Sender: TObject);
    procedure TCPServerConnectionRequest(Sender: TObject;
      requestID: Integer);
    procedure TCPServerDataArrival(Sender: TObject; bytesTotal: Integer);
    procedure TCPServerError(Sender: TObject; Number: Smallint;
      var Description: WideString; Scode: Integer; const Source,
      HelpFile: WideString; HelpContext: Integer;
      var CancelDisplay: WordBool);
    procedure TCPClientConnect(Sender: TObject);
    procedure TCPClientError(Sender: TObject; Number: Smallint;
      var Description: WideString; Scode: Integer; const Source,
      HelpFile: WideString; HelpContext: Integer;
      var CancelDisplay: WordBool);
    procedure TCPClientDataArrival(Sender: TObject; bytesTotal: Integer);
    procedure SendMessage(const Msg:String);
    procedure PostMessage(const Msg:String);
    procedure InternetBoardMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure InternetbtnDonePlacingClick(Sender: TObject);
    procedure DisconnectError();
    procedure CannotConnectError();

    //ADDED BY RANDY
    procedure DeclareLoss(WhichPlayer:Integer);

    //ENCODE/DECODE
    procedure DecodeMove(DataString:String);
    procedure DecodeMessage(DataString:String);
    procedure DecodePlace(DataString:String);
    procedure DecodeOther(DataString:String);
    procedure EncodeMove(WhosTurn,X,Y:Integer);
    procedure EncodePlace(WhichPlayer:Integer; ShipType : TShip; SX,SY,EX,EY:Integer);

    //SAVE / LOAD GAME
    procedure SaveGame1Click(Sender: TObject);

    procedure MediaPlayer1Enter(Sender: TObject);
    procedure MediaPlayer1Notify(Sender: TObject);
    procedure LoadGame1Click(Sender: TObject);
    procedure LoadGame(SaveGame : TGame);
    function SaveGame():TGame;
    function SaveCellState(val:TCellState):Byte;
    function SaveShipType(val:TShip):Byte;
    function LoadCellState(val:byte):TCellState;
    function LoadShipType(val:byte):TShip;

     //highscore procedures
    procedure fillscorearray();
    procedure writehighScores();
    procedure searchScores(score : integer;WhichPlayer : Integer);
    procedure TopScores1Click(Sender: TObject);

    //Random Placing
    procedure RandomGenerate();
    procedure TryPlacing(WhichMap, Size: integer; ship: TShip);

    private
    { Private declarations }
    public
    { Public declarations }
        Board : TBattleBoard; //game board.  Contains player1 and player2 board
    end;
var
    FrmBattleShip: TFrmBattleShip;
    FrmNewGame: TFrmNewGame;

    PlaceProcess: Boolean = True; //placement mode
    Play: Boolean = False;        //play mode
    InProgress: Boolean = False;  //placement in progress?
    CellOccupied: Boolean = False;//temp, used for placement

    //true when ship is on board. Used for placement only.
    ACarrierExist: Boolean = False;
    BattleShipExist: Boolean = False;
    CarrierExist: Boolean = False;
    SubmarineExist: Boolean = False;
    DestroyerExist: Boolean = False;

    CurrentShipPlaced : TShip; //current ship being placed
    CurStartLocX: Integer; //startloc x for current ship placed
    CurStartLocY: Integer; //startloc y for current ship placed
    Count: Integer = 1; //the number of ships placed

    PlayerName : array[1..2] of String; //player names
    CurrentPlayer : Integer=1; //current player number
    OpposingPlayer : Integer=2; //opposing player number
    PlayerBoatFlags : array[1..2] of TBoatFlags; //Boolean info on sunk boats etc...

    GameMode : TGameMode;

    //INTERNET ADD-ON
    Player1Ready : Boolean = False; // player1 placed?
    Player2Ready : Boolean = False; // player2 placed?
    ShowAll : Boolean = False;
    HideBoats : Boolean = False;
    FreezeGame : Boolean = False;

    NS_ACarrierPic, NS_CarrierPic, NS_BattleshipPic, NS_DestroyerPic, NS_SubmarinePic : TBitmap;
    WE_ACarrierPic, WE_CarrierPic, WE_BattleshipPic, WE_DestroyerPic, WE_SubmarinePic : TBitmap;
    HitPic, PlacementPic, MissPic : TBitmap;

    //variable2 for game score
    player1numShots: integer;
    player2numShots: integer;
    player1Score: integer;
    player2Score: integer;
    ScoresFile: textfile;
    overwrite: bool;
    placing : integer;

    top_scores : array[1..10] of Thighscore;

    // Random Placements
    RCol: integer; // randomly generated column
    RRow: integer; // randomly generated row
    RDirection: integer; // direction indicated by a number


implementation

uses
    Internet,Chat,highscore, newHighScoreinput;

{$R *.DFM}

//////////////////////////////////////////////////////////////////
// BoardMouseDown - Handles all occurances of clicks on the game
//                  board.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.BoardMouseDown(Sender: TObject;
    Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    Col, Row: Integer;
    Msg : PChar;
    dup : Boolean;
begin
    // exit the game if it freezes
    if(FreezeGame)then Exit;

    //If player is placing ships
    if PlaceProcess then
    begin
    if CurrentShipPlaced <> sNone then
    begin
        CurrentMouseToCell(X,Y,Col,Row);
        StatusBar1.SimpleText := PlayerName[CurrentPlayer] + ' select the end point for your ' + GetShipName(CurrentShipPlaced) + '.';
        PlaceBoat(CurrentPlayer, Col, Row);
        PaintGame();
    end
    else
        StatusBar1.SimpleText := PlayerName[CurrentPlayer] + ' you must select a ship first!';
    end;

    //if game in session then handle click.
    if Play then
    begin
        OpposingMouseToCell(X,Y,Col,Row); //Get Shot coords

    //Player 1 clicks on own board.
    if(Sender = Board1) AND (CurrentPlayer = 1) then
    begin
        // do nothing
    end
    //Player 2 fires on Player 1.
    else if(Sender = Board1) AND (CurrentPlayer = 2) And (GameMode <> gmSinglePlayer) then
    begin
        if(Board[1,Col,Row].State=csDefault)then
        begin
            CannonFired(1,Col,Row);
            CheckBoard(1);//Check for sink or win
            if(GameMode = gmMultiplayer) then
            begin
                HideBoats := True;
                PaintGame(); //paints states only
                Msg := PChar(PlayerName[1] + ' are you ready?');
                Application.MessageBox(Msg,'End Turn',MB_OK);
                HideBoats := False;
            end;
            EndTurn();
        end
        else //duplicate shot
            StatusBar1.SimpleText :='You have already shot there!';
    end
    //Player 1 fires on Player 2.
    else if(Sender = Board2) AND (CurrentPlayer = 1) then
    begin
        dup := false;
        if(Board[2,Col,Row].State=csDefault)then
        begin
            CannonFired(2,Col,Row);
            CheckBoard(2);//Check for sink or win
            if(GameMode = gmMultiplayer) then
            begin
                HideBoats := True;
                PaintGame(); //paints states only
                Msg := PChar(PlayerName[2] + ' are you ready?');
                Application.MessageBox(Msg,'End Turn',MB_OK);
                HideBoats := False;
            end;
            EndTurn();
        end
        else  //duplicate shot
        begin
            StatusBar1.SimpleText:='You have already shot there!';
            dup := true;
        end;

        if (GameMode = gmSinglePlayer) and not dup then
            DoComputerTurn();
    end
    //Player 2 clicks on own board.
    else if(Sender = Board2) AND (CurrentPlayer = 2) then
        begin
         // do nothing
        end;
    end;

    PaintGame() //Repaint game boards for current player
end;

//////////////////////////////////////////////////////////////////
// CannonFired - Fires on a given map at position Col, Row
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.CannonFired(WhichMap, Col, Row: Integer);
begin
    PlaySound(FIRE_WAV,0,SND_SYNC);

    if Board[WhichMap, Col, Row].Ship <> sNone then
    begin
        Board[WhichMap, Col, Row].State := csHit;
        StatusBar1.SimpleText := 'Hit!';
        //Play the sound for a hit
        PlaySound(HIT_WAV,0,SND_ASYNC);

        if (Board[1,Col, Row].State = csHit) Then
            player2score := player2score + 1
        else
            player1score := player1score + 1;
    end
    else
    begin
        Board[WhichMap, Col, Row].State := csMiss;
        StatusBar1.SimpleText := 'Missed.';
        //Play the sound for miss
        PlaySound(MISS_WAV,0,SND_ASYNC);

        if (Board[1,Col, Row].State = csMiss) Then
            player2score := player2score -1
        else
            player1score := player1score -1;
    end;

    PaintCellStates(WhichMap);
end;

//////////////////////////////////////////////////////////////////
// CheckBoard - Checks for sunk boats and for victory.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.CheckBoard(WhichMap:Integer);
begin
    with PlayerBoatFlags[WhichMap] do
    begin
        //check for and set sink.
        if not ACarrierSunk then
            ACarrierSunk := CheckSink(WhichMap,sACarrier);

        if not CarrierSunk then
            CarrierSunk := CheckSink(WhichMap,sCarrier);

        if not BattleShipSunk then
        begin
            BattleShipSunk := CheckSink(WhichMap,sBattleShip);
            if BattleShipSunk then
                //Play the sound for a sunk battleship
                PlaySound(SUNK_BATTLESHIP_WAV, 0, SND_ASYNC);
        end;

        if not SubmarineSunk then
            SubmarineSunk := CheckSink(WhichMap,sSubmarine);

        if not DestroyerSunk then
            DestroyerSunk := CheckSink(WhichMap,sDestroyer);

        //check for win
        if ACarrierSunk and CarrierSunk and BattleShipSunk and SubmarineSunk and DestroyerSunk then
        begin
            if(WhichMap=CurrentPlayer) then //CurrentPlayer loses
                DeclareLoss(CurrentPlayer)
            else //Current Player wins
                DeclareVictory(CurrentPlayer);
        end;
    end;
end;

//////////////////////////////////////////////////////////////////
// CheckSink - Checks to see if a boat is sunk
//////////////////////////////////////////////////////////////////
function TFrmBattleShip.CheckSink(WhichMap:Integer;WhichShip:TShip):Boolean;
var
    SX,SY,EX,EY : Integer; //start and end points
    Col,Row : Integer;
    Bearing :TBearing;

label
    FoundShip;

begin
    //find ship
    for Col := 0 to NItems-1 do
        for Row := 0 to NItems-1 do
        begin
            if(Board[WhichMap,Col,Row].Ship = WhichShip) then
                goto FoundShip;
        end;

        FoundShip:

        //set start and end points
        with Board[WhichMap,Col,Row].ShipLoc do
        begin
            SX := StartPosX;
            SY := StartPosY;
            EX := EndPosX;
            EY := EndPosY;
        end;

        Bearing := GetBearing(SX,SY,EX,EY);
        case Bearing of

        bNorth:
        begin
            for Row := SY downto EY do
                if(Board[Whichmap,SX,Row].State <> csHit) then
                begin
                    Result := False; //NOT SUNK
                    Exit;
                end;
        end;

        bSouth:
        begin
            for Row := SY to EY do
                if(Board[Whichmap,SX,Row].State <> csHit) then
                begin
                    Result := False; //NOT SUNK
                    Exit;
                end;
        end;

        bEast:
        begin
            for Col := SX to EX do
                if(Board[Whichmap,Col,SY].State <> csHit) then
                    begin
                        Result := False; //NOT SUNK
                        Exit;
                    end;
        end;

        bWest:
        begin
            for Col := SX downto EX do
                if(Board[Whichmap,Col,SY].State <> csHit) then
                begin
                    Result := False; //NOT SUNK
                    Exit;
                end;
        end;
        end; //end case

    //check for sink
    Result:=True; //Boat Sunk!
    StatusBar1.SimpleText := PlayerName[CurrentPlayer] + ' you sunk ' + PlayerName[OpposingPlayer] + 's ' + GetShipName(WhichShip) + '!';
    //Play sound when the board is set
    PlaySound(SUNK_BOAT_WAV, 0, SND_ASYNC);
end;

//////////////////////////////////////////////////////////////////
// DeclareVictory - Declares vitory for a given player
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.DeclareVictory(WhichPlayer : Integer);
var
    winner_score: integer;
    score: string;
begin
    PlaySound(WIN_WAV,0,SND_SYNC);
    Application.MessageBox(PChar(PlayerName[WhichPlayer] + ' you are the winner!'),'Congratulations',MB_OK);

    if(WhichPlayer = 1)then
        winner_score := player1score * 1000
    else
        winner_score := player2score * 1000;

    if (winner_score < 0) then
        winner_score := 0;

    score := inttostr(winner_score);
    ShowMessage(score);
    searchScores(strtoint(score), WhichPlayer);
    ResetGame();
    ShowAll := True; //Shows both players ships
end;

//////////////////////////////////////////////////////////////////
// PlaceBoat - Places CurrentShipPlaced(GLOBAL VAR) on a given
//             board at position Col, Row
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PlaceBoat(WhichMap, Col, Row: Integer);
var
    Size : Integer;
begin
    if Board[WhichMap, Col, Row].Ship = sNone then
    begin
        Size := SizeSelect(); //Get size of boat

    if Not InProgress then //Place new boat
    begin
        ClearPlacements(WhichMap);
        PlaceInProgress(WhichMap, Col, Row, Size);
    end
    else if InProgress then //Finish previous placement
        if Board[WhichMap, Col, Row].State = csPlacement then //finish
        begin
            SetEndPt(WhichMap, Col, Row,CurrentShipPlaced); //places boat...
            ClearPlacements(WhichMap);
            StatusBar1.SimpleText := 'Click on the menu to place your ship(s).  Select Done when finished.';
        end

    else //Replace boat...
    begin
        ClearPlacements(WhichMap);
        PlaceInProgress(WhichMap, Col, Row, Size);
    end;
    end;
end;

//////////////////////////////////////////////////////////////////
// PlaceInProgress - Places a new ship.  Triggers place InProgress
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PlaceInProgress(WhichMap, Col, Row, Size: Integer);
begin
    CurStartLocX := Col;
    CurStartLocY := Row;
    SetPlacements(WhichMap, Col, Row, Size);
    InProgress := True;
end;

//////////////////////////////////////////////////////////////////
// SetPlacements - Checks and sets valid placement cells.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.SetPlacements(WhichMap, Col, Row, Size: Integer);
begin
    if (Col-Size) >= 0 then  //Check West
    begin
        CheckHorizontal(WhichMap, Row, Col-Size, Col);
        if not CellOccupied then
            Board[WhichMap, Col-Size, Row].State := csPlacement;
        CellOccupied := False;
    end;

    if (Col+Size) < NItems then //Check East
    begin
        CheckHorizontal(WhichMap, Row, Col, Col+Size);
        if not CellOccupied then
            Board[WhichMap, Col+Size, Row].State := csPlacement;
        CellOccupied := False;
    end;

    if (Row-Size) >= 0 then  //Check North
    begin
        CheckVertical(WhichMap, Col, Row-Size, Row);
        if not CellOccupied then
            Board[WhichMap, Col,Row-Size].State := csPlacement;
        CellOccupied := False;
    end;

    if (Row+Size) < NItems then //Check South
    begin
        CheckVertical(WhichMap, Col, Row, Row+Size);
        if not CellOccupied then
            Board[WhichMap, Col,Row+Size].State := csPlacement;
        CellOccupied := False;
    end;
end;

//////////////////////////////////////////////////////////////////
// ClearPlacements - Replaces all occurances of csPlacement in
//                   WhichMap to csDefault.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.ClearPlacements(WhichMap : Integer);
var
    C, R : Integer;
begin
    for C := 0 to NItems - 1 do
        for R := 0 to NItems - 1 do
        begin
            if(Board[WhichMap,C,R].State = csPlacement) then
                Board[WhichMap,C,R].State := csDefault;
        end;
end;

//////////////////////////////////////////////////////////////////
// PaintGame - Paints the 2 game boards.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PaintGame();
begin
    if(PlaceProcess) then
    begin
        ErasePaint(1);
        ErasePaint(2);
    end
    else
    begin
        if(CurrentPlayer=1) then
            ErasePaint(2)
        else
            ErasePaint(1);
    end;

    if(ShowAll) then //paint everything, done at game over.
    begin
        PaintShips(1);
        PaintShips(2);
        PaintCellStates(1);
        PaintCellStates(2);
    end
    else if(HideBoats) then //just paint states
    begin
        PaintCellStates(1);
        PaintCellStates(2);
    end
    else if(GameMode = gmInternet) then //Internet game mode
    begin
        if (TCPServer.Enabled) then// SERVERS TURN
        begin
            PaintShips(1);
            PaintCellStates(1);

            if not PlaceProcess then
            begin
                PaintCellStates(2);
            end;
        end
        else if(TCPClient.Enabled) then //CLIENTS TURN
        begin
            PaintShips(2);
            PaintCellStates(2);
            begin
                PaintCellStates(1);
            end;
        end
        else //ERROR ??? not connected
            StatusBar1.SimpleText := 'ERROR: Connection problem.';
    end
    else  //not internet game mode.
    begin
        if (GameMode = gmSinglePlayer) and not ShowAll then
        begin
            PaintShips(1);
            PaintCellStates(1);
            PaintCellStates(2);
        end
        else
        begin
            PaintShips(CurrentPlayer);
            PaintCellStates(CurrentPlayer);
            PaintCellStates(OpposingPlayer);
        end;
    end;
end;

//////////////////////////////////////////////////////////////////
// ErasePaint - Clears any paint on the canvas of a given board.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.ErasePaint(WhichMap : Integer);
var
    C, R : Integer;
begin

    for C := 0 to NItems - 1 do             //paint all white
        for R := 0 to NItems - 1 do
            begin
                ColorCell(WhichMap, C, R, clWhite);
            end;

    if WhichMap = 1 then                 //redraw gridlines
        Board1.Repaint
    else
        Board2.Repaint;
end;

//////////////////////////////////////////////////////////////////
// PaintCellStates - Paints the hits, misses and placment bitmaps.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PaintCellStates(WhichMap : Integer);
var
    C, R : Integer;
    CellCoords : TRect;
begin

    for C := 0 to NItems - 1 do
    begin
        for R := 0 to NItems - 1 do
        begin
            if(Board[WhichMap,C,R].State) <> csDefault then //paint only the cells with pics
            begin
                if(WhichMap=1)then //get pic coords
                    CellCoords := Board1.CellRect(C,R)
                else
                    CellCoords := Board2.CellRect(C,R);

                case(Board[WhichMap,C,R].State) of //paint cell
                    csHit: PaintPic(WhichMap,CellCoords,HitPic);
                    csMiss: PaintPic(WhichMap,CellCoords,MissPic);
                    csPlacement : PaintPic(WhichMap,CellCoords,PlacementPic);
                end; //end case
            end;
        end; //end for R
    end; //end for C
end;

//////////////////////////////////////////////////////////////////
// PaintShips - Finds all valid start points and paints the ship
//              located at that position.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PaintShips(WhichMap : Integer);
var
    C, R : Integer;
begin
    for C := 0 to NItems - 1 do
        for R := 0 to NItems - 1 do
        begin
            //check start point of cell
            if(Board[WhichMap,C,R].ShipLoc.StartPosX = C) AND (Board[WhichMap,C,R].ShipLoc.StartPosY = R) then
            PaintShip(WhichMap,C,R); //paint boat if start point found.
        end;
end;

//////////////////////////////////////////////////////////////////
// PaintShip - Paints a ship at a given position to a given map.
//             Called by PaintShips, which paints ALL ships.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PaintShip(WhichMap,Col,Row : Integer);
var
    PicCoords : TRect;
    PicTopLeft : TPoint;
    PicBotRight : TPoint;
    Bearing : TBearing;
begin

    //Find the direction the boat is facing
    with Board[WhichMap,Col,Row].ShipLoc do
        Bearing := GetBearing(StartPosX,StartPosY,EndPosX,EndPosY);

        case Bearing of
        bEast, bSouth:
        begin
            //Set Top Left
            PicCoords := Board1.CellRect(Col,Row);
            PicTopLeft.x := PicCoords.TopLeft.x;
            PicTopLeft.y := PicCoords.TopLeft.y;

            //Set Bottom Right
            PicCoords := Board1.CellRect(Board[WhichMap,Col,Row].ShipLoc.EndPosX  ,Board[WhichMap,Col,Row].ShipLoc.EndPosY);
            PicBotRight.x := PicCoords.BottomRight.x;
            PicBotRight.y := PicCoords.BottomRight.y;
         end;

        bWest,bNorth:
        begin
            //Set top left
            PicCoords := Board1.CellRect(Board[WhichMap,Col,Row].ShipLoc.EndPosX  ,Board[WhichMap,Col,Row].ShipLoc.EndPosY);
            PicTopLeft.x := PicCoords.TopLeft.x;
            PicTopLeft.y := PicCoords.TopLeft.y;

            //Set bot right
            PicCoords := Board1.CellRect(Col,Row);
            PicBotRight.x := PicCoords.BottomRight.x;
            PicBotRight.y := PicCoords.BottomRight.y;

            //Set finished picture coords
        end;
        end; //end case

        //Set the finished Picture Coords
        PicCoords.TopLeft.x := PicTopLeft.x;
        PicCoords.TopLeft.y := PicTopLeft.y;
        PicCoords.BottomRight.x := PicBotRight.x;
        PicCoords.BottomRight.y := PicBotRight.y;

        //LoadPic for boat
        case Bearing of
        bNorth,bSouth:
        begin
            case Board[WhichMap,Col,Row].Ship of
                sACarrier: PaintPic(WhichMap,PicCoords,NS_ACarrierPic);
                sBattleship: PaintPic(WhichMap,PicCoords,NS_BattleshipPic);
                sSubmarine: PaintPic(WhichMap,PicCoords,NS_SubmarinePic);
                sCarrier: PaintPic(WhichMap,PicCoords,NS_CarrierPic);
                sDestroyer: PaintPic(WhichMap,PicCoords,NS_DestroyerPic);
            end;
        end ;

        bEast, bWest:
        begin
            case Board[WhichMap,Col,Row].Ship of
                sACarrier: PaintPic(WhichMap,PicCoords,WE_ACarrierPic);
                sBattleship: PaintPic(WhichMap,PicCoords,WE_BattleshipPic);
                sSubmarine: PaintPic(WhichMap,PicCoords,WE_SubmarinePic);
                sCarrier: PaintPic(WhichMap,PicCoords,WE_CarrierPic);
                sDestroyer: PaintPic(WhichMap,PicCoords,WE_DestroyerPic);
            end;
        end ;
    end;
end;

//////////////////////////////////////////////////////////////////
// PaintPic- draws the pictures on the approprite board canvas
//
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PaintPic(WhichMap : Integer; Coords : TRect; Pic : TBitmap);
begin
    if(WhichMap = 1) then
    begin
        Board1.Canvas.Brush.Bitmap := Pic;
        Board1.Canvas.StretchDraw(Coords, Pic);
    end
    else begin
        Board2.Canvas.Brush.Bitmap := Pic;
        Board2.Canvas.StretchDraw(Coords, Pic);
    end;
end;

//////////////////////////////////////////////////////////////////
// GetBearing - Gets the bearing from a start and end position.
//              Returns: bNorth,bEast,bSouth or bWest.
//////////////////////////////////////////////////////////////////
function TFrmBattleShip.GetBearing(Stx,Sty,Endx,Endy : Integer):TBearing;
var
    Bearing : TBearing;
begin
    if (StX < EndX) then
        Bearing := bEast
    else if (StX > EndX) then
        Bearing := bWest
    else if (StY < EndY) then
        Bearing := bSouth
    else
        Bearing := bNorth;

    Result := Bearing; //return bearing
end;

//////////////////////////////////////////////////////////////////
// SetEndPt - Stores the end point and calls place boat.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.SetEndPt(WhichMap, Col, Row: Integer; Ship:TShip);
begin
    InProgress := False;
    Board[WhichMap,Col,Row].ShipLoc.EndPosX := Col;
    Board[WhichMap,Col,Row].ShipLoc.EndPosY := Row;

    with Board[WhichMap,Col,Row].ShipLoc do
        BoatPlaced(WhichMap,CurStartLocX,CurStartLocY,EndPosX,EndPosY,Ship); //places boat
end;

//////////////////////////////////////////////////////////////////
// BoatPlaced - Places boat in specified board.
//////////////////////////////////////////////////////////////////
//BoatPlaced(Vars[0],Vars[2],Vars[3],Vars[4],Vars[5],Ship);
procedure TFrmBattleShip.BoatPlaced(WhichMap,StartPtX,StartPtY,EndPtX,EndPtY: Integer; Ship:TShip);
begin
    if StartPtX = EndPtX then
    begin  // vertical
        if StartPtY > EndPtY then     // facing north
            PlaceVertical(WhichMap, StartPtX, EndPtY, StartPtY,Ship)
        else                          // facing south
            PlaceVertical(WhichMap, StartPtX, StartPtY, EndPtY,Ship);
    end
    else
    begin   // horizontal
        if StartPtX < EndPtX then    // facing east
            PlaceHorizontal(WhichMap, StartPtY, StartPtX, EndPtX,Ship)
        else                         // facing west
            PlaceHorizontal(WhichMap, StartPtY, EndPtX, StartPtX,Ship);
    end;

    if(GameMode = gmInternet) then //send place to opponent
    begin
        if (TCPServer.Enabled and (WhichMap=1)) or (TCPClient.Enabled and (WhichMap=2)) then//placed on your map
        begin
            EncodePlace(WhichMap,Ship,StartPtX,StartPtY,EndPtX,EndPtY);
            SuccessPlace(Ship); //only places into your map.
        end;
    end
    else
        SuccessPlace(Ship);

end;

//////////////////////////////////////////////////////////////////
// PlaceVertical - Helper function for BoatPlaced.  Places a boat
//                 vertically in a specified board.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PlaceVertical(WhichMap, A, B, C: Integer; Ship:TShip);
var
    I: Integer;
begin
    for I := B to C do
    begin
        Board[WhichMap, A, I].Ship := Ship;

        with Board[WhichMap, A, I].ShipLoc do
        begin
            StartPosX := A;
            StartPosY := B;
            EndPosX := A;
            EndPosY := C;
        end;
    end;
end;

//////////////////////////////////////////////////////////////////
// PlaceHorizontal - Helper function for BoatPlaced.  Places a
//                   boat horizontally in a specified board.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PlaceHorizontal(WhichMap, A, B, C: Integer; Ship:TShip);
var
    I: Integer;
begin
    for I := B to C do
    begin
        Board[WhichMap, I, A].Ship := Ship;

        with Board[WhichMap, I, A].ShipLoc do
        begin
            StartPosX := B;
            StartPosY := A;
            EndPosX := C;
            EndPosY := A;
        end;
    end;
end;

//////////////////////////////////////////////////////////////////
// SuccessPlace -  Wraps up after a successful place.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.SuccessPlace(Ship:TShip);
begin
    case Ship of

        sACarrier:
        begin
            ACarrierExist := True;
            btnACarrier.enabled := False;
        end;

        sBattleship:
        begin
            BattleShipExist := True;
            btnBattleShip.enabled := False;
        end;

        sCarrier:
        begin
            CarrierExist := True;
            btnCarrier.enabled := False;
        end;

        sSubmarine:
        begin
            SubmarineExist := True;
            btnSubmarine.enabled := False;
        end;

        sDestroyer:
        begin
            DestroyerExist := True;
            btnDestroyer.enabled := False;
        end;
    end;

    CurrentShipPlaced := sNone;
end;

//////////////////////////////////////////////////////////////////
// CheckHorizontal -  Checks for boats horizontally. Sets a global
//                    flag if boat is found.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.CheckHorizontal(WhichMap, Row, Col1, Col2: Integer);
var
    I : Integer;
begin
    for I := Col1 to Col2 do
        if Board[WhichMap, I, Row].Ship <> sNone then
            CellOccupied := True;
end;

//////////////////////////////////////////////////////////////////
// CheckVertical -  Checks for boats vertically. Sets a global
//                    flag if boat is found.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.CheckVertical(WhichMap, Col, Row1, Row2: Integer);
var
    I : Integer;
begin
    for I := Row1 to Row2 do
        if Board[WhichMap, Col, I].Ship <> sNone then
            CellOccupied := True;
end;

//////////////////////////////////////////////////////////////////
// ColorCell - Colors a specific cell of a specific board a
//             specified color.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.ColorCell(WhichMap, Col, Row: Integer;
                                        color: TColor);
var
    CellRectangle : TRect;
    Dgrid : TDrawGrid;
begin
    if WhichMap = 1 then
        Dgrid := Board1
    else
        Dgrid := Board2;

    Dgrid.Canvas.Brush.Color := color;
    CellRectangle := Dgrid.CellRect(Col, Row);
    CellRectangle.Bottom := CellRectangle.Bottom + Board1.GridLineWidth;
    CellRectangle.Right := CellRectangle.Right + Board1.GridLineWidth;
    Dgrid.Canvas.FillRect(CellRectangle);
end;

//////////////////////////////////////////////////////////////////
// Menu Options - option in the menu bar
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.Quit1Click(Sender: TObject);
begin
    Close;
end;

//////////////////////////////////////////////////////////////////
// NewGame - Starting the new game
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.NewGame1Click(Sender: TObject);
begin
    // reinitialize and repaint
    StartNewGame();
end;

//////////////////////////////////////////////////////////////////
// About1Click - show the about information
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.About1Click(Sender: TObject);
begin
    MessageDlg ('Mastering Delphi BattleShip',
    mtInformation, [mbOK], 0);
end;

//////////////////////////////////////////////////////////////////
// btnPlaceShip - Event called when place ship button is pressed.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.btnPlaceShipClick(Sender: TObject);
begin
    if(FreezeGame) then Exit;

    if not InProgress then //no place in progress
    begin
        if(Sender = btnACarrier) then
            CurrentShipPlaced := sACarrier
        else if(Sender = btnBattleship) then
            CurrentShipPlaced := sBattleship
        else if(Sender = btnCarrier) then
            CurrentShipPlaced := sCarrier
        else if(Sender = btnSubmarine) then
            CurrentShipPlaced := sSubmarine
        else if(Sender = btnDestroyer) then
            CurrentShipPlaced := sDestroyer;

        if(GameMode<>gmInternet) then
            StatusBar1.SimpleText := PlayerName[CurrentPlayer] + ' click on the grid to place your ' + GetShipName(CurrentShipPlaced) + '.'
        else
        begin
            if(TCPClient.Enabled) then
                StatusBar1.SimpleText := PlayerName[2] + ' click on the grid to place your ' + GetShipName(CurrentShipPlaced) + '.'
            else
                StatusBar1.SimpleText := PlayerName[1] + ' click on the grid to place your ' + GetShipName(CurrentShipPlaced) + '.';
        end;
    end
    else //Place InProgress
    begin
        InProgress := False; //End place in progress
        ClearPlacements(1);
        ClearPlacements(2);
        PaintGame();
    end;
end;

//////////////////////////////////////////////////////////////////
// FormCreate - Event ran at startup.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.FormCreate(Sender: TObject);
begin
    randomize;
    //load pic for aircraft carrier NS
    NS_ACarrierPic := TBitmap.Create;
    NS_ACarrierPic.LoadFromFile(NS_ACARRIER_PIC);
    NS_ACarrierPic.TransparentColor := NS_ACarrierPic.Canvas.Pixels[0,0];
    NS_ACarrierPic.Transparent := True;

    //load pic for carrier NS
    NS_CarrierPic:= TBitmap.Create;
    NS_CarrierPic.LoadFromFile(NS_CARRIER_PIC);
    NS_CarrierPic.TransparentColor := NS_CarrierPic.Canvas.Pixels[0,0];
    NS_CarrierPic.Transparent := True;

    //load pic for destroyer NS
    NS_DestroyerPic:= TBitmap.Create;
    NS_DestroyerPic.LoadFromFile(NS_DESTROYER_PIC);
    NS_DestroyerPic.TransparentColor := NS_DestroyerPic.Canvas.Pixels[0,0];
    NS_DestroyerPic.Transparent := True;

    //load pic for submarine NS
    NS_SubmarinePic := TBitmap.Create;
    NS_SubmarinePic.LoadFromFile(NS_SUBMARINE_PIC);
    NS_SubmarinePic.TransparentColor := NS_SubmarinePic.Canvas.Pixels[0,0];
    NS_SubmarinePic.Transparent := True;

    //load pic for battleship NS
    NS_BattleshipPic := TBitmap.Create;
    NS_BattleshipPic.LoadFromFile(NS_BATTLESHIP_PIC);
    NS_BattleshipPic.TransparentColor := NS_BattleshipPic.Canvas.Pixels[0,0];
    NS_BattleshipPic.Transparent := True;

    //load picture of aircraft carrier WE
    WE_ACarrierPic:= TBitmap.Create;
    WE_ACarrierPic.LoadFromFile(WE_ACARRIER_PIC);
    WE_ACarrierPic.TransparentColor := WE_ACarrierPic.Canvas.Pixels[0,0];
    WE_ACarrierPic.Transparent := True;

    //load picture of carrier WE
    WE_CarrierPic:= TBitmap.Create;
    WE_CarrierPic.LoadFromFile(WE_CARRIER_PIC);
    WE_CarrierPic.TransparentColor := WE_CarrierPic.Canvas.Pixels[0,0];
    WE_CarrierPic.Transparent := True;

    //load picture of destroyer WE
    WE_DestroyerPic := TBitmap.Create;
    WE_DestroyerPic.LoadFromFile(WE_DESTROYER_PIC);
    WE_DestroyerPic.TransparentColor := WE_DestroyerPic.Canvas.Pixels[0,0];
    WE_DestroyerPic.Transparent := True;

    //load picture of a submarine WE
    WE_SubmarinePic := TBitmap.Create;
    WE_SubmarinePic.LoadFromFile(WE_SUBMARINE_PIC);
    WE_SubmarinePic.TransparentColor := WE_SubmarinePic.Canvas.Pixels[0,0];
    WE_SubmarinePic.Transparent := True;

    //load picture of a battleship WE
    WE_BattleshipPic := TBitmap.Create;
    WE_BattleshipPic.LoadFromFile(WE_BATTLESHIP_PIC);
    WE_BattleshipPic.TransparentColor := WE_BattleshipPic.Canvas.Pixels[0,0];
    WE_BattleshipPic.Transparent := True;

    //load picture for a hit
    HitPic := TBitmap.Create;
    HitPic.LoadFromFile(HIT_PIC);
    HitPic.TransparentColor := HitPic.Canvas.Pixels[0,0];
    HitPic.Transparent := True;

    //load picture of the placment node
    PlacementPic := TBitmap.Create;
    PlacementPic.LoadFromFile(PLACEMENT_PIC);
    PlacementPic.TransparentColor := PlacementPic.Canvas.Pixels[0,0];
    PlacementPic.Transparent := True;

    //load  picture of the miss picture
    MissPic := TBitmap.Create;
    MissPic.LoadFromFile(MISS_PIC);
    MissPic.TransparentColor := MissPic.Canvas.Pixels[0,0];
    MissPic.Transparent := True;

    ResetGame(); //Init game variables;
    HideGame;
    //load the game music
    MediaPlayer.play;

    fillscorearray();
end;

//////////////////////////////////////////////////////////////////
// SizeSelect - Returns the size of the CurrentShipPlaced (GLOBAL)
//////////////////////////////////////////////////////////////////
function TFrmBattleShip.SizeSelect(): Integer;
begin
    case CurrentShipPlaced of
        sACarrier: Result := 4;
        sBattleship: Result := 3;
        sCarrier, sSubmarine: Result := 2;
        sDestroyer: Result := 1;
        else Result := 0;
    end;
end;

//////////////////////////////////////////////////////////////////
// btnDonePlacingClick
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.btnDonePlacingClick(Sender: TObject);
var
    status : Boolean;
begin
    if(FreezeGame) then Exit;

    status := CheckStatus();

    if not status then
        StatusBar1.SimpleText := 'You must place ALL your ships.'
    else
    begin
        StatusBar1.SimpleText := PlayerName[OpposingPlayer] + ' place your ships.';
        count := count + 1;

        if Count <= 2 then
        begin
            Board1.Hide;
            ShowButtons();

            if GameMode = gmSinglePlayer then
            begin
                RandomPlace(2); // place random boats for computer
                btnDonePlacingClick(Sender);
            end
            else
                Board2.Show;
        end
        else if Count > 2 then
        begin
            Board2.Hide;
            btnDonePlacing.Visible := False;
            btnRandomPlace.Visible := False;
            btnPlay.Enabled := True;
            StatusBar1.SimpleText := 'Press Play to start the game.';
        end;

        EndTurn();
    end;
end;

//////////////////////////////////////////////////////////////////
// CheckStatus - Check to see if all players ships are placed.
//////////////////////////////////////////////////////////////////
function TFrmBattleShip.CheckStatus(): Boolean;
begin
    if ACarrierExist And BattleShipExist And CarrierExist And SubmarineExist And DestroyerExist then
    begin
        Result := True;
    end
    else
        Result := False;
end;

//////////////////////////////////////////////////////////////////
// ShowButtons - Enables all the ship placement buttons.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.ShowButtons();
begin
    btnACarrier.enabled := True;
    btnBattleship.enabled := True;
    btnCarrier.enabled := True;
    btnSubmarine.enabled := True;
    btnDestroyer.enabled := True;
    ACarrierExist := False;
    CarrierExist := False;
    BattleshipExist := False;
    SubmarineExist := False;
    DestroyerExist := False;
end;

//////////////////////////////////////////////////////////////////
// btnPlayClick - OnClickEvent for play button.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.btnPlayClick(Sender: TObject);
begin
    if(FreezeGame) then Exit;

    PlaceProcess:= False;
    Play := True;
    btnPlay.Hide;
    //play the openning music
    PlaySound(START_WAV,0,SND_SYNC);
    StatusBar1.SimpleText := 'Let us Play';
    Board1.Show;
    Board2.Show;
    PaintGame();
    //allow the user to save the current game
    SaveGame1.Enabled := true;
end;

//////////////////////////////////////////////////////////////////
// EndTurn - End CurrentPlayers turn and move on to next player
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.EndTurn();
begin
    if(CurrentPlayer = 1) then //switch to player two
    begin
        CurrentPlayer := 2;
        OpposingPlayer := 1;
        ShapeP2.Show;
        ShapeP1.Hide;
    end
    else //switch to player one
    begin
        CurrentPlayer := 1;
        OpposingPlayer := 2;
        ShapeP1.Show;
        ShapeP2.Hide;
    end;
end;

//////////////////////////////////////////////////////////////////
// CurrentMouseToCell - MouseToCell for CurrentPlayer's grid.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.CurrentMouseToCell(X,Y:Integer; var ACol,ARow : Integer);
begin
  if(CurrentPlayer = 1) then
        Board1.MouseToCell(X,Y,ACol,ARow)
  else
        Board2.MouseToCell(X,Y,ACol,ARow);
end;

//////////////////////////////////////////////////////////////////
// OpposingMouseToCell - MouseToCell for OpposingPlayer's grid.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.OpposingMouseToCell(X,Y:Integer; var ACol,ARow : Integer);
begin
    if(CurrentPlayer = 1) then
        Board2.MouseToCell(X,Y,ACol,ARow)
    else
        Board1.MouseToCell(X,Y,ACol,ARow);
end;

//////////////////////////////////////////////////////////////////
// BoardSelectCell - Gets rid of selection...
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.BoardSelectCell(Sender: TObject; ACol,
    ARow: Integer; var CanSelect: Boolean);
begin
    CanSelect := False;
end;

//////////////////////////////////////////////////////////////////
// GetShipName - Returns a srting for a given ship type
//////////////////////////////////////////////////////////////////
function TFrmBattleShip.GetShipName(ShipType : TShip) : String;
begin
    case ShipType of
         sACarrier:   Result := 'Aircraft Carrier';
         sBattleShip: Result := 'Battleship';
         sCarrier:    Result := 'Carrier';
         sSubmarine:  Result := 'Submarine';
         sDestroyer:  Result := 'Destroyer';
         sNone:       Result := 'None!';
    end;
end;

//////////////////////////////////////////////////////////////////
// FormPaint - Repaints game when entering from another window.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.FormPaint(Sender: TObject);
begin
    PaintGame(); //Draws board based on CurrentPlayer (GLOBAL)
end;

//////////////////////////////////////////////////////////////////
// StartNewGame - Starts a new game different ways, depending on
//                GameMode (GLOBAL). Also prompt for player names
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.StartNewGame();
begin
    try //show NewGame form.
        FrmNewGame := TFrmNewGame.Create(FrmBattleShip);
        FrmNewGame.ShowModal;
        if(FrmNewGame.CancelPressed)then
            Exit;

        if Play or InProgress or Player1Ready or Player2Ready then
            if(Application.MessageBox('Are you sure.  This will end your current game?','Notice',MB_YESNO) = 7) then Exit; //User pressed NO
                GameMode := TGameMode(FrmNewGame.GameMode);
                FreezeGame := False;

    finally
        FrmNewGame.Free;
    end;

    case GameMode of //Prompt for player names and call subfunctions

        gmSinglePlayer:
        begin
            ResetGame();
            StatusBar1.SimpleText := 'Single Player Game';
            PlayerName[1] := InputBox('Please enter your name: ','','Player1');
            PlayerName[2] := 'Battle Bob';
            StartSinglePlayerGame();
        end;

        gmMultiPlayer:
        begin
            ResetGame();
            StatusBar1.SimpleText := 'Multi Player Game';
            PlayerName[1] := 'Player1'; //DefaultNames
            PlayerName[2] := 'Player2';
            PlayerName[1] := InputBox('Please enter player ones name: ','','Player1');
            PlayerName[2] := InputBox('Please enter player twos name: ','','Player2');
            StartMultiPlayerGame();
        end;

        gmInternet:
        begin
            ResetGame();
            HideGame();
            PlayerName[1] := 'Player1'; //DefaultNames
            PlayerName[2] := 'Player2';
            StatusBar1.SimpleText := 'Internet Game';
            StartInternetGame();
        end;
    end; //end case
end;

//////////////////////////////////////////////////////////////////
// StartSinglePlayerGame - Init single player game.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.StartSinglePlayerGame();
begin
    ShowGame(); //unhide game
    Board2.Hide; //hide opponents board
    ShapeP1.Show;

    Board1.OnMouseDown := BoardMouseDown;
    Board2.OnMouseDown := BoardMouseDown;
end;

//////////////////////////////////////////////////////////////////
// Dummy - dummy event to block any event
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.Dummy(Sender: TObject;
    Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
;
end;

//////////////////////////////////////////////////////////////////
// StartMultiPlayerGame - Init multi player game.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.StartMultiPlayerGame();
begin
    ShowGame(); //unhide game
    ShapeP1.Show;
    Board2.Hide; //Hide player2 board
    CurrentShipPlaced := sNone; //NoShipPlaced
    StatusBar1.SimpleText := PlayerName[1] + ' place your ships.';

    //events changed by internet game
    Board1.OnMouseDown := BoardMouseDown;
    Board2.OnMouseDown := BoardMouseDown;
    btnDonePlacing.OnClick := BtnDonePlacingClick;
end;

//////////////////////////////////////////////////////////////////
// StartInternetGame - Init internet game.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.StartInternetGame();
begin
    Internet.frmInternet.ShowModal; //host / join new game
    if(Internet.frmInternet.CancelPressed) then
        Exit;

    ShowGame(); //unhide game boards
    btnChat.Visible := True; //unhide chat button

    //Change any events to speacial ones for multiplayer.
    Board1.OnMouseDown := InternetBoardMouseDown;
    Board2.OnMouseDown := InternetBoardMouseDown;
    btnDonePlacing.OnClick := InternetBtnDonePlacingClick;

    //hide opponent board and start placing ships...
    if(TCPServer.Enabled)then
    begin
        Board2.Hide;
        ShapeP1.Show;
    end
    else
    begin
        Board1.Hide;
        ShapeP2.Show;
    end;
end;

//////////////////////////////////////////////////////////////////
// ResetGame - Resets all game variables.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.ResetGame();
var
    I,J,K : Integer;
begin
    for I := 1 to 2 do //Init Boards
        for J := 0 to NItems - 1 do
            for K := 0 to NItems - 1 do
            begin
                Board[I, J, K].Ship := sNone;
                Board[I, J, K].State := csDefault;
                Board[I, J, K].ShipLoc.StartPosX := -1;
                Board[I, J, K].ShipLoc.StartPosY := -1;
                Board[I, J, K].ShipLoc.EndPosX := -1;
                Board[I, J, K].ShipLoc.EndPosY := -1;
            end;

    ShowButtons();

    btnPlay.Enabled := False;
    btnChat.Visible := False; //hide chat button

    for I := 1 to 2 do//reset PlayerBoatFlags
    begin
        PlayerBoatFlags[I].ACarrierSunk := False;
        PlayerBoatFlags[I].BattleShipSunk := False;
        PlayerBoatFlags[I].CarrierSunk := False;
        PlayerBoatFlags[I].SubmarineSunk := False;
        PlayerBoatFlags[I].DestroyerSunk := False;
    end;

    PlaceProcess:=True; //placement mode
    Play:=False;        //play mode
    InProgress:= False;  //placement in progress?
    CellOccupied:= False;//temp, used for placement

    //true when ship is on board. Used for placement only.
    ACarrierExist:= False;
    BattleShipExist:= False;
    CarrierExist:= False;
    SubmarineExist:= False;
    DestroyerExist:= False;

    CurrentShipPlaced := sNone; //current ship being placed
    Count:= 1; //the number of ships placed

    CurrentPlayer :=1; //current player number
    OpposingPlayer :=2; //opposing player number

    //INTERNET ADD-ON
    Player1Ready := False; // player1 placed?
    Player2Ready := False; // player2 placed?
    ShowAll := False;
    HideBoats := False;
    FreezeGame := False;

    player1score := 0;
    player2score := 0;
end;

//////////////////////////////////////////////////////////////////
// HideGame - Hides the game.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.HideGame();
begin
    ShapeP1.Visible := False;
    ShapeP2.Visible := False;
    btnDonePlacing.Visible := False;
    btnRandomPlace.Visible := False;
    btnPlay.Visible := False;
    gbPlace.Visible := False;
    btnACarrier.Visible := False;
    btnBattleShip.Visible := False;
    btnCarrier.Visible := False;
    btnSubmarine.Visible := False;
    btnDestroyer.Visible := False;
    lblPlayer1.Visible := False;
    lblPlayer2.Visible := False;
    Board1.Visible := False;
    Board2.Visible := False;
    StatusBar1.SimpleText := 'Click on File or press F8 to start new game.';
end;

//////////////////////////////////////////////////////////////////
// ShowGame - Shows the game.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.ShowGame();
begin
    btnDonePlacing.Visible := True;
    btnRandomPlace.Visible := True;
    btnPlay.Visible := True;
    gbPlace.Visible := True;
    btnACarrier.Visible := True;
    btnBattleShip.Visible := True;
    btnCarrier.Visible := True;
    btnSubmarine.Visible := True;
    btnDestroyer.Visible := True;

    lblPlayer1.Caption := PlayerName[1];
    lblPlayer1.Visible := True;

    lblPlayer2.Caption := PlayerName[2];
    lblPlayer2.Visible := True;

    Board1.Visible := True;
    Board2.Visible := True;
end;

//////////////////////////////////////////////////////////////////
// btnRandomPlaceClick - Place ships randomly.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.btnRandomPlaceClick(Sender: TObject);
begin
    // call function to randomly place the ships on the board
    RandomPlace(CurrentPlayer);
    if GameMode = gmSinglePlayer then
        btnRandomPlace.Visible := False; // disable the button
end;

//////////////////////////////////////////////////////////////////
// RandomGenerate - generate random numbers
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.RandomGenerate();
begin
    randomize;

    // generate random number to be used as column
    RCol := Random (NItems);

    // generate random number to be used as row
    RRow := Random (NItems);

    // generate a number between 0 to 3 for each direction
    RDirection := Random (4);
end;

//////////////////////////////////////////////////////////////////
// RandomPlace - Clear and place ships randomly on a given board
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.RandomPlace(WhichMap:Integer);
var
    Size: Integer;
    ship : Tship;
begin
    // check to see that the aircrafe carrier doesn't exist
    if Not ACarrierExist then
    begin
        RandomGenerate(); // generate random numbers for row and col
        ship := sACarrier; // pick the ship type
        CurrentShipPlaced := ship;  // set the current ship
        size := SizeSelect();  // get the size of the current ship

        // try until the ship is successfully placed
        while Not ACarrierExist do
        begin
            RandomGenerate();
            TryPlacing(WhichMap, Size, Ship);
            // set the flag to successful placment
            if (Board[WhichMap, RCol, RRow].Ship = ship) then
                ACarrierExist := True;
        end;
   end;

    // check to see that the battleship doesn't exist
    if Not BattleShipExist then
    begin
        RandomGenerate();   // generate random numbers for row and col
        ship := sBattleShip;  // pick the ship type
        CurrentShipPlaced := ship;   // set the current ship to get the size
        size := SizeSelect();    // get the size of the current ship

        // try until the ship is successfully placed
        while Not BattleShipExist do
        begin
            RandomGenerate();
            TryPlacing(WhichMap, Size, Ship);
            // set the flag to successful placment
            if (Board[WhichMap, RCol, RRow].Ship = ship) then
                BattleShipExist := True;
        end;
   end;

    // check to see that the carrier doesn't exist
    if Not CarrierExist then
    begin
        RandomGenerate();   // generate random numbers for row and col
        ship := sCarrier;   // pick the ship type
        CurrentShipPlaced := ship; // set the current ship to get the size
        size := SizeSelect();

        // try until the ship is successfully placed
        while Not CarrierExist do
        begin
            RandomGenerate();
            TryPlacing(WhichMap, Size, Ship);
            // set the flag to successful placment
            if (Board[WhichMap, RCol, RRow].Ship = ship) then
                CarrierExist := True;
        end;
    end;

    // check to see that the submarine doesn't exist
    if Not SubmarineExist then
    begin
        RandomGenerate();   // generate random numbers for row and col
        ship := sSubmarine; // pick the ship type
        CurrentShipPlaced := ship; // set the current ship to get the size
        size := SizeSelect();

        // try until the ship is successfully placed
        while Not SubmarineExist do
        begin
            RandomGenerate();
            TryPlacing(WhichMap, Size, Ship);
            // set the flag to successful placment
            if (Board[WhichMap, RCol, RRow].Ship = ship) then
                SubmarineExist := True;
        end;
    end;

    // check to see that the destroyer doesn't exist
    if Not DestroyerExist then
    begin
        RandomGenerate();   // generate random numbers for row and col
        ship := sDestroyer; // pick the ship type
        CurrentShipPlaced := ship;  // set the current ship to get the size
        size := SizeSelect();

        // try until the ship is successfully placed
        while Not DestroyerExist do
        begin
            RandomGenerate();
            TryPlacing(WhichMap, Size, Ship);
            // set the flag to successful placment
            if (Board[WhichMap, RCol, RRow].Ship = ship) then
                DestroyerExist := True;
        end;
    end;

    // done placing all the boats
end;

//////////////////////////////////////////////////////////////////
// TryPlacing - get a coordinate and a direction and try placing
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.TryPlacing(WhichMap, Size: integer; ship: Tship);
begin
    // make sure the coordinate on the board is empty
    if Board[WhichMap, RCol, RRow].Ship = sNone then
    begin
        case RDirection of

            0:// Place South
            begin
                // make sure that the possible size is within the board
                if ((RRow+Size)<=(NItems-1))then
                begin
                    // check that the another ship is not blocking its path
                    CheckVertical(WhichMap, RCol, RRow, RRow+Size);
                    // Place the boat if the cells are empty
                    if Not CellOccupied then
                    begin
                        BoatPlaced(WhichMap, RCol, RRow, RCol, RRow+Size, ship);
                        paintship(WhichMap, RCol, RRow);
                    end;
                    CellOccupied := False;
                end;
            end;

            1:// Place North
            begin
                // make sure that the possible size is within the board
                if ((RRow-Size)>=0) then
                begin
                    // check that the another ship is not blocking its path
                    CheckVertical(WhichMap, RCol, RRow-Size, RRow);
                    // Place the boat if the cells are empty
                    if Not CellOccupied then
                    begin
                        BoatPlaced(WhichMap, RCol, RRow-Size, RCol, RRow, ship);
                        paintship(WhichMap, RCol, RRow-Size);
                    end;
                    CellOccupied := False;
                end;
            end;

            2:// Place East
            begin
                // make sure that the possible size is within the board
                if ((RCol+Size)<=(NItems-1)) then
                begin
                    // check that the another ship is not blocking its path
                    CheckHorizontal(WhichMap, RRow, RCol, RCol+Size);
                    // Place the boat if the cells are empty
                    if Not CellOccupied then
                    begin
                        BoatPlaced(WhichMap, RCol, RRow, RCol+Size, RRow, ship);
                        paintship(WhichMap, RCol, RRow);
                    end;
                    CellOccupied := False;
                end;
            end
        else // West
            begin
                // make sure that the possible size is within the board
                if ((RCol-Size)>=0) then
                begin
                    // check that the another ship is not blocking its path
                    CheckHorizontal(WhichMap, RRow, RCol-Size, RCol);
                    // Place the boat if the cells are empty
                    if Not CellOccupied then
                    begin
                        BoatPlaced(WhichMap, RCol-Size, RRow, RCol, RRow, ship);
                        paintship(WhichMap, RCol-Size, RRow);
                    end;
                    CellOccupied := False;
                end;
            end;

        end; // end of the case
    end; // end of the original if statment

end;

//////////////////////////////////////////////////////////////////
// DoComputerTurn - Computer takes a turn.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.DoComputerTurn();
var
    shootpt : TPoint;
    I, X, Y, PrevShipLeft : integer;
    Up, Down, Left, Right : TCellState;
    selected, UpFlag, DownFlag, LeftFlag, RightFlag : Boolean;
begin
    // initialize all the flags
    UpFlag:=True;
    DownFlag:=True;
    LeftFlag:=True;
    RightFlag:=True;
    selected := False;

    // loop twice: once for continuous hits and the other for a single hit
    for I:=1 to 2 do
        // run thru columns
        for X:=0 to (NItems-1) do
        if not Selected then // flag to block any useless loops
            // run thru rows
            for Y:=0 to (NItems-1) do
            if not Selected then  // flag to block any useless loops
            begin
                // make sure the cell is hit
                if Board[1, X, Y].State = csHit then
                begin

                    // check the cell above
                    if ((Y+1)>(NItems-1)) then
                        UpFlag := False
                    else
                        Up:= Board[1, X, (Y+1)].State;

                    // check the cell below
                    if ((Y-1)<0) then
                        DownFlag := False
                    else
                        Down:=Board[1, X, (Y-1)].State;

                    // check the cell to the right
                    if ((X+1)>(NItems-1)) then
                        RightFlag := False
                    else
                        Right:=Board[1, X+1, Y].State;

                    // check the cell to the left
                    if ((X-1)<0) then
                        LeftFlag := False
                    else
                        Left:=Board[1, X-1, Y].State;

                    if (I = 1) then  // look for two consecutive hits
                    begin
                        // ship to the left of current position
                        if (Right=csHit) And (Left=csDefault) And LeftFlag then
                        begin
                            ShootPt.X := X - 1;
                            ShootPt.Y := Y;
                            selected := True;
                        end
                        // ship to the right of the current position
                        else if (Left=csHit) And (Right=csDefault) And RightFlag then
                        begin
                            ShootPt.X := X + 1;
                            ShootPt.Y := Y;
                            selected := True;
                        end
                        // ship above the current position
                        else if (Up=csHit) And (Down=csDefault) And DownFlag then
                        begin
                            ShootPt.Y := Y - 1;
                            ShootPt.X := X;
                            selected := True;
                        end
                        // ship below the current position
                        else if (Down=csHit) And (Up=csDefault) And UpFlag then
                        begin
                            ShootPt.Y := Y + 1;
                            ShootPt.X := X;
                            selected := True;
                        end;
                    end
            else   // look for 1 hit
            begin
                // make sure that the cell you are looking at is not the
                // boat already sank
                if Not ((right=csHit) or (Left=csHit) or (Up=csHit) or (Down=csHit)) then
                    if (Right=csDefault) then
                    begin
                        ShootPt.X := X + 1;
                        ShootPt.Y := Y;
                        selected := True;
                    end
                    else if (Left=csDefault) then
                    begin
                        ShootPt.X := X - 1;
                        ShootPt.Y := Y;
                        selected := True;
                    end
                    else if (Up=csDefault) then
                    begin
                        ShootPt.Y := Y + 1;
                        ShootPt.X := X;
                        selected := True;
                    end
                    else if (Down=csDefault) then
                    begin
                        ShootPt.Y := Y - 1;
                        ShootPt.X := X;
                        selected := True;
                    end;
                end;
            end;
        end;

    // no shots found
    If Not Selected then
    begin
        // take random shots hopefully diagonal
        shootpt := GetShotCoord();
    end;

    // make the shot with the coordinates found
    CannonFired(1, shootpt.X, shootpt.Y);

    EndTurn(); // end the turn for the computer
end;

//////////////////////////////////////////////////////////////////
// GetShotCoord - Gets a coord from board 1 for the computer to
//                shoot at. Help function for DoComputerTurn()
//////////////////////////////////////////////////////////////////
function TFrmBattleShip.GetShotCoord() : TPoint;
var
    RPoint : TPoint;
    cellst : TCellState;
begin
    RandomGenerate;
    RPoint.X := RCol;
    RPoint.Y := RRow;

    cellst := Board[1, RPoint.X, RPoint.Y].State;

    // make sure that the random number was not previously chosen
    While (cellst = csHit) Or (cellst = csMiss) do
    begin
        // generate new set of col and row
        RandomGenerate;
        RPoint.X := RCol;
        RPoint.Y := RRow;
        cellst := Board[1, RPoint.X, RPoint.Y].State;
    end;

    result:= RPoint;   //return the points found
end;

//////////////////////////////////////////////////////////////////
// SaveGame1Click - Save a battleship game
//
///////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.SaveGame1Click(Sender: TObject);
var
    Outfile: file of TGame;
    SavedGame : TSavedGame;
begin
    SavedGame.SavedGame := SaveGame();  //save nesc. data

    if SaveDialog.Execute then
    begin
        //Name the file
        AssignFile(Outfile, SaveDialog.filename);

        //Create the file to accept records of TSaveGame size
        Rewrite(Outfile{, sizeof(TGame)});

        //Write the board contents of both boards to file
        BlockWrite(Outfile, SavedGame.SavedGame, 1);

        //Close the file
        CloseFile(Outfile);
        PostMessage('File saved as: ' + SaveDialog.Filename);
    end;
end;

//////////////////////////////////////////////////////////////////
// LoadGame1Click - Load a battleship game onto the screen
//
///////////////////////////////////////////////////////////////////
//load game should be enabled only when no boats have been placed//
procedure TFrmBattleShip.LoadGame1Click(Sender: TObject);
var
    Infile: file of TGame;
    SavedGame : TSavedGame;
begin
    //Request the name of the file to open
    if (OpenDialog.Execute) then begin
        //Name the file
        AssignFile(Infile, opendialog.filename);

        //Create the file to accept records of cell size
        Reset(Infile{,sizeof(TGame)});

        PostMessage('FilePos at open: ' + IntToStr(FilePos(Infile)));
        //Write the file contents to the board
        BlockRead(Infile, SavedGame.SavedGame, 1);

        PostMessage('FilePos at end of read: ' + IntToStr(FilePos(Infile)));
        //Close the file
        CloseFile(Infile);

        //Lock 'n Load!
        LoadGame(SavedGame.SavedGame);
    end;
end;

//////////////////////////////////////////////////////////////////
// SaveGame1Click - Gets a coord from board 1 for the computer to
//                shoot at. Help function for DoComputerTurn()
///////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.LoadGame(SaveGame : TGame);

var
    Player,Col,Row : Byte;
    //CellVar        |  Value
    //*****************************
    // 0 - ShipType  |  byte (0-sNone, 1-ACarrier,2-Carrier,3-Battleship,4-Destroyer,5-Submarine)
    // 1 - State     |  byte (0-csDefault,1-csHit,2-csMiss,3-csPlacement)
    // 2 - SX        |  byte
    // 3 - SY        |  byte
    // 4 - EX        |  byte
    // 5 - EY        |  byte
    //*****************************
begin
    //game board
    for Player := 1 to 2 do
        for Col := 0 to (nitems-1) do
            for Row:= 0 to (nitems-1) do
            begin
                Board[Player,Col,Row].Ship  := LoadShipType(SaveGame.BoardArray[Player,Col,Row,0]);
                Board[Player,Col,Row].State := LoadCellState(SaveGame.BoardArray[Player,Col,Row,1]);
                Board[Player,Col,Row].ShipLoc.StartPosX  := SaveGame.BoardArray[Player,Col,Row,2];
                Board[Player,Col,Row].ShipLoc.StartPosY := SaveGame.BoardArray[Player,Col,Row,3];
                Board[Player,Col,Row].ShipLoc.EndPosX := SaveGame.BoardArray[Player,Col,Row,4];
                Board[Player,Col,Row].ShipLoc.EndPosY := SaveGame.BoardArray[Player,Col,Row,5];
            end;

    //PlayerBoatFlags
    for Player := 1 to 2 do
    begin
        PlayerBoatFlags[Player].ACarrierSunk  := SaveGame.PlayerFlagArray[Player,0];
        PlayerBoatFlags[Player].CarrierSunk  := SaveGame.PlayerFlagArray[Player,1];
        PlayerBoatFlags[Player].BattleshipSunk  := SaveGame.PlayerFlagArray[Player,2];
        PlayerBoatFlags[Player].SubmarineSunk  := SaveGame.PlayerFlagArray[Player,3];
        PlayerBoatFlags[Player].DestroyerSunk  := SaveGame.PlayerFlagArray[Player,4];
    end;

    //PlayerNames
    PlayerName[1] := SaveGame.PlayerName[1]; //Player 1 name.
    PlayerName[2] := SaveGame.PlayerName[2]; //Player 2 name.

    //GameMode
    case SaveGame.GameModeByte of
        0: GameMode := gmSinglePlayer;
        1: GameMode := gmMultiPlayer;
        2: GameMode := gmInternet;
    end;

    PlaceProcess := False; //ships must be placed before you can save...
    InProgress := False;
    CurrentShipPlaced := sNone;

    Play := SaveGame.Play; //is the game in play mode?

    //Count := 1; ?? not sure

    CurrentPlayer := SaveGame.CurrentPlayer; //Whos turn
    OpposingPlayer := SaveGame.OpposingPlayer; //NOT Whos turn

    ShowAll := SaveGame.ShowAll;   //paints all maps
    HideBoats := SaveGame.HideBoats;  //hides both players boats, used for player transition in
                                      // gmMultiplayer.
    FreezeGame := SaveGame.FreezeGame; //Freezes internet game on connection error

    Player1Ready := SaveGame.Player1Ready;  //have players done placing in Internet Mode?
    Player2Ready := SaveGame.Player2Ready;

    ShowGame();

    btnACarrier.Enabled := False;  //disable placement buttons
    btnCarrier.Enabled := False;
    btnDestroyer.Enabled := False;
    btnSubmarine.Enabled := False;
    btnBattleship.Enabled := False;

    btnPlay.Visible := False; //hide the play button
    btnRandomPlace.Visible := False; //hide the 'Place For Me' button.

    if(GameMode=gmInternet) then
        btnChat.Visible := True
    else
        btnChat.Visible := False;

    PostMessage('Game Loaded.');
    PaintGame() //load 'er up!
end;

//////////////////////////////////////////////////////////////////
// SaveGame - encode the saving information
///////////////////////////////////////////////////////////////////
function TFrmBattleShip.SaveGame():TGame;
var
    Player,Col,Row: Byte;
    //CellVar        |  Value
    //*****************************
    // 0 - ShipType  |  byte (0-sNone, 1-ACarrier,2-Carrier,3-Battleship,4-Destroyer,5-Submarine)
    // 1 - State     |  byte (0-csDefault,1-csHit,2-csMiss,3-csPlacement)
    // 2 - SX        |  byte
    // 3 - SY        |  byte
    // 4 - EX        |  byte
    // 5 - EY        |  byte
    //*****************************
begin
    PostMessage('Saving Game...');

    //game board
    for Player := 1 to 2 do
        for Col := 0 to (nitems-1) do
            for Row:= 0 to (nitems-1) do
            begin
                Result.BoardArray[Player,Col,Row,0] := SaveShipType(Board[Player,Col,Row].Ship);
                Result.BoardArray[Player,Col,Row,1] := SaveCellState(Board[Player,Col,Row].State);
                Result.BoardArray[Player,Col,Row,2] := Board[Player,Col,Row].ShipLoc.StartPosX;
                Result.BoardArray[Player,Col,Row,3] := Board[Player,Col,Row].ShipLoc.StartPosY;
                Result.BoardArray[Player,Col,Row,4] := Board[Player,Col,Row].ShipLoc.EndPosX;
                Result.BoardArray[Player,Col,Row,5] := Board[Player,Col,Row].ShipLoc.EndPosY;
            end;

    //PlayerBoatFlags[]
    for Player := 1 to 2 do
    begin
        Result.PlayerFlagArray[Player,0] := PlayerBoatFlags[Player].ACarrierSunk;
        Result.PlayerFlagArray[Player,1] := PlayerBoatFlags[Player].CarrierSunk;
        Result.PlayerFlagArray[Player,2] := PlayerBoatFlags[Player].BattleshipSunk;
        Result.PlayerFlagArray[Player,3] := PlayerBoatFlags[Player].DestroyerSunk;
        Result.PlayerFlagArray[Player,4] := PlayerBoatFlags[Player].SubmarineSunk;
    end;

    Result.PlayerName[1] := PlayerName[1];
    Result.PlayerName[2] := PlayerName[2];

    //(GameModeByte=> 0-gmSinglePlayer,1-gmMultiPlayer,2-gmInternet)
    case GameMode of
        gmSinglePlayer: Result.GameModeByte := 0;
        gmMultiPlayer:  Result.GameModeByte := 1;
        gmInternet:     Result.GameModeByte := 2;
    end;

    Result.CurrentPlayer := CurrentPlayer;
    Result.OpposingPlayer := OpposingPlayer;
    Result.Play := Play;                             //game in play?
    Result.Player1Ready := Player1Ready;             //gmInternet only
    Result.Player2Ready := Player2Ready;             //gmInternet only
    Result.ShowAll := ShowAll;                       //show both boards at victory
    Result.HideBoats := HideBoats;                   //show no boats, used for player transition
    Result.FreezeGame := FreezeGame;                 //gmInternet freezes on connection error

    PostMessage('Game Saved.');
end;

//////////////////////////////////////////////////////////////////
// LoadCellState - Decode the saved information
///////////////////////////////////////////////////////////////////
function TFrmBattleShip.LoadCellState(val : byte): TCellState;
begin
    // State     |  byte (0-csDefault,1-csHit,2-csMiss,3-csPlacement)
    case val of
        0: Result := csDefault;
        1: Result := csHit;
        2: Result := csMiss;
        3: Result := csPlacement;
    end;
end;

//////////////////////////////////////////////////////////////////
// SaveCellState - decode the saving information
///////////////////////////////////////////////////////////////////
function TFrmBattleShip.SaveCellState(val :TCellState): Byte ;
begin
    // State     |  byte (0-csDefault,1-csHit,2-csMiss,3-csPlacement)
    case val of
        csDefault:    Result := 0;
        csHit:        Result := 1;
        csMiss:       Result := 2;
        csPlacement:  Result := 3;
    end;
end;

//////////////////////////////////////////////////////////////////
// LoadShipType - decode the saving information
///////////////////////////////////////////////////////////////////
function TFrmBattleShip.LoadShipType(val : byte) : TShip;
begin
    // ShipType  |  byte (0-sNone, 1-ACarrier,2-Carrier,3-Battleship,4-Destroyer,5-Submarine)
    case val of
        0: Result := sNone;
        1: Result := sACarrier;
        2: Result := sCarrier;
        3: Result := sBattleship;
        4: Result := sDestroyer;
        5: Result := sSubmarine;
    end;
end;

//////////////////////////////////////////////////////////////////
// SaveShipType - encode the saving information
///////////////////////////////////////////////////////////////////
function TFrmBattleShip.SaveShipType(val : TShip) : Byte;
begin
    // ShipType  |  byte (0-sNone, 1-ACarrier,2-Carrier,3-Battleship,4-Destroyer,5-Submarine)
    case val of
        sNone:        Result := 0;
        sACarrier:    Result := 1;
        sCarrier:     Result := 2;
        sBattleship:  Result := 3;
        sDestroyer:   Result := 4;
        sSubmarine:   Result := 5;
    end;
end;

//////////////////////////////////////////////////////////////////
// Functions Added by Randy
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.DeclareLoss(WhichPlayer:Integer);
begin
    PlaySound(LOSE_WAV,0,SND_SYNC);
    Application.MessageBox(PChar(PlayerName[WhichPlayer] + ' you have been defeated!'),'Message',MB_OK);
    ShowAll := True; //Shows both players ships
end;

//////////////////////////////////////////////////////////////////
// Randy's Internet Functions
// ==================================================
// Allows multiplayer game over the internet using winsock
//
//
//
//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
// DecodeData - Decodes data rec'd
// ================================
// =MSG=MessageText=
// =MOV=1=X=Y=
// =PLC=1=A=SX=SY=EX=EY=
// =OTH=CMD=  R->Ready  N->Name
//
//
// sACarrier: ShipStr := '1';
// sCarrier: ShipStr := '2';
// sBattleship: ShipStr := '3';
// sSubmarine: ShipStr := '4';
// sDestroyer: ShipStr := '5';
/////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.DecodeData(Data:OLEVariant);
var
        DataString : String; //data rec'd
        Command : array[0..3] of Char;
        Operands : array of Char;
        i,j : Integer;
begin
        DataString := VarToStr(Data);

        //PostMessage('Decoding : '+ DataString);

        i:=2;
        j:=0;
        while (DataString[i] <> '=') and (j < 3)do
         begin
                 Command[j] := DataString[i];
                 i := i+1;
                 j := j+1;
         end;

        Command[j] :=''#0;//terminate string

        //PostMessage('CMD : |' +String(Command) +'|');

        SetLength(Operands,Length(DataString));
        //Strip commandstring from...
        j:=0;
        while DataString[i] <> ''#0 do
         begin
                 Operands[j] := DataString[i];
                 i := i+1;
                 j := j+1;
         end;

        Operands[j] := ''#0; //terminate

        //PostMessage('OPS: |' + String(Operands)+'|');

        if(Command = 'MSG') then
                DecodeMessage(String(Operands))
        else if(Command = 'PLC') then
                DecodePlace(String(Operands))
        else if(Command = 'MOV') then
                DecodeMove(String(Operands))
        else if(Command = 'OTH') then
                DecodeOther(String(Operands));
end;

//////////////////////////////////////////////////////////////////
// SendData - Sends a DataString to other player
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.SendData(const DataString: String);
var
        Data : OLEVariant;
begin
        Data := DataString; //store data packet

        try
          if(TCPServer.Enabled)then //Send data packet
                TCPServer.SendData(Data)
          else if(TCPClient.Enabled)then
                TCPClient.SendData(Data)
          else
                DisconnectError();
        except
          //error
          DisconnectError();
        end;

end;

//////////////////////////////////////////////////////////////////
// TCPServerConnectionRequest - Server accepts client.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.TCPServerConnectionRequest(Sender: TObject;
  requestID: Integer);
begin
        Internet.frmInternet.Label2.Caption := 'Please wait...';
        Internet.frmInternet.Label3.Visible := False;
        if TCPServer.State <> sckClosed then TCPServer.Close;
        TCPServer.Accept(requestID);
        Chat.frmChat.StatusBar1.SimpleText := 'Online';
        //send the client the server players name
        SendData('=OTH=N='+ BattleShip.PlayerName[1] + '=');
        Internet.frmInternet.Close;
end;

//////////////////////////////////////////////////////////////////
// TCPClientConnect - When client connects to server.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.TCPClientConnect(Sender: TObject);
begin
        Internet.frmInternet.Close; //close window
        Chat.frmChat.StatusBar1.SimpleText := 'Online';
        //send the server the client players name
        SendData('=OTH=N='+ BattleShip.PlayerName[2] + '=');
end;

//////////////////////////////////////////////////////////////////
// ErrorEvents - Handle winsock errors
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.TCPServerError(Sender: TObject; Number: Smallint;
  var Description: WideString; Scode: Integer; const Source,
  HelpFile: WideString; HelpContext: Integer; var CancelDisplay: WordBool);
begin
        Internet.frmInternet.StatusBar1.SimpleText := IntToStr(Number) + '  -  ' + Description;
        DisconnectError();
end;
procedure TFrmBattleShip.TCPClientError(Sender: TObject; Number: Smallint;
  var Description: WideString; Scode: Integer; const Source,
  HelpFile: WideString; HelpContext: Integer; var CancelDisplay: WordBool);
begin
        Internet.frmInternet.StatusBar1.SimpleText := IntToStr(Number) + '  -  ' + Description;
        if(Number = 10061)then
                CannotConnectError()
        else
                DisconnectError();
end;

//////////////////////////////////////////////////////////////////
// DisconnectError - Thrown when players get disconnected.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.DisconnectError();
begin
        Beep;
        Application.MessageBox('Connection problem detected.  You are no longer connected.','Error',MB_OK);
        Chat.frmChat.StatusBar1.SimpleText := 'Disconnected';
        if TCPServer.State <> sckClosed then TCPServer.Close;
        if TCPClient.State <> sckClosed then TCPClient.Close;
        StatusBar1.SimpleText := 'Connection Problem : Disconnected!';
        FreezeGame := True;
end;

//////////////////////////////////////////////////////////////////
// CannotConnectError - Thrown when players get disconnected.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.CannotConnectError();
begin
        Beep;
        Application.MessageBox('Cannot connect.  Invalid IP, or opponent is not available.','Error',MB_OK);
        Chat.frmChat.StatusBar1.SimpleText := 'Disconnected';
        if TCPServer.State <> sckClosed then TCPServer.Close;
        if TCPClient.State <> sckClosed then TCPClient.Close;
        StatusBar1.SimpleText := 'Connection Problem : Cannot connect!';
        FreezeGame := True;
end;

//////////////////////////////////////////////////////////////////
// OnDataArrival - DecodeData on arrival
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.TCPClientDataArrival(Sender: TObject;
  bytesTotal: Integer);
var
        data : OLEVariant;
begin
        TCPClient.GetData(data,varOLEStr);
        DecodeData(data);
end;
procedure TFrmBattleShip.TCPServerDataArrival(Sender: TObject;
  bytesTotal: Integer);
var
        data : OLEVariant;
begin
        TCPServer.GetData(data,varOLEstr);
        DecodeData(data);
end;

//////////////////////////////////////////////////////////////////
// SendMessage - Sends a message to the other player and posts
//               message to chat board.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.SendMessage(const Msg:String);
var
        Name : String;
        SMsg : String;
begin

        if(TCPServer.Enabled)then
                Name := PlayerName[1]
        else
                Name := PlayerName[2];

        SMsg := '=MSG=' + Name + ' says: '#13#10'' + Msg + '=';
        PostMessage(Name + ' says: '#13#10'' + Msg);
        SendData(SMsg);  //send to opponent for decode.
end;

//////////////////////////////////////////////////////////////////
// PostMessage - Posts message to chat board
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.PostMessage(const Msg:String);
begin
        Chat.frmChat.MemoWindow.Lines.Add(Msg);
        if not Chat.frmChat.Visible then
                Chat.frmChat.Show;
        Chat.frmChat.BringToFront;
end;
procedure TFrmBattleShip.btnChatClick(Sender: TObject);
begin
        Chat.frmChat.Show;
end;

procedure TFrmBattleShip.DecodeMove(DataString:String);
var
        WhichPlayer : Integer;
        Move : TPoint;
        CurVar : array[0..2] of Char;
        i,j : Integer;

begin
        //=1=X=Y=

        i := 2;
        j := 0;
        while(DataString[i] <> '=') do //GET WHICHPLAYER
         begin
                CurVar[j] := DataString[i];
                j := j+1;
                i := i+1;
         end;

        CurVar[j] := ''#0; //terminate
        WhichPlayer := StrToInt(String(CurVar)); //store
        j:=0;
        i := i+1; //skip '='

        while(DataString[i] <> '=') do //GET X
         begin
                CurVar[j] := DataString[i];
                j := j+1;
                i := i+1;
         end;
        CurVar[j] := ''#0; //terminate
        Move.X := StrToInt(String(CurVar)); //store
        j:=0;
        i := i+1; //skip '='

        while(DataString[i] <> '=') and (DataString[i] <> ''#0) do //GET Y
         begin
                CurVar[j] := DataString[i];
                j := j+1;
                i := i+1;
         end;
        CurVar[j] := ''#0; //terminate
        Move.Y := StrToInt(String(CurVar)); //store

        //shot was fired BY whichplayer
        //so swap
        if WhichPlayer = 1 then
                WhichPlayer := 2
        else
                WhichPlayer := 1;

        CannonFired(WhichPlayer,Move.X,Move.Y);
        EndTurn();
end;

procedure TFrmBattleShip.DecodePlace(DataString:String);
var
        i,j,k : Integer;
        Vars : array[0..5] of Integer;
        CurVar : array[0..2] of Char;
        Ship :TShip; //Ship type placed
begin
        //=1=2=SX=SY=EX=EY=

        i := 2; //start point for DataString scan
        k := 0; //index in int array
        while(DataString[i] <> ''#0) do //scan DataString
        begin
                j := 0; //reset curvar index
                while(DataString[i] <> '=') do //Get CurVar
                 begin
                        CurVar[j] := DataString[i];
                        j := j+1;
                        i := i+1;
                 end;

                CurVar[j] := ''#0; //terminate
                Vars[k] := StrToInt(String(CurVar)); //store
                k := k+1; //next var
                i := i+1; //skip '='
        end;

        case Vars[1] of
          1: Ship := sACarrier;
          2: Ship := sCarrier;
          3: Ship := sBattleship;
          4: Ship := sSubmarine;
          5: Ship := sDestroyer;
        end;

        BoatPlaced(Vars[0],Vars[2],Vars[3],Vars[4],Vars[5],Ship);
        PaintGame();
end;
procedure TFrmBattleShip.DecodeOther(DataString:String);
var
        Ops : array[0..1,0..30] of Char;
        i,j,CurOp : Integer;
begin
        //=R=                   READY
        //=N=1=Battle Bob=      PLAYER NAME
        CurOp := 0;
        i := 2; //scan DataString
        while(DataString[i] <> ''#0) do //scan DataString for Ops
        begin
                j := 0; //reset curvar index
                while(DataString[i] <> '=') do //Get CurVar
                 begin
                        Ops[CurOp,j] := DataString[i];
                        j := j+1;
                        i := i+1;
                 end;

                Ops[CurOP,j] := ''#0; //terminate
                CurOp := CurOp+1; //next var
                i := i+1; //skip '='
        end;

        case Ops[0,0] of //Char
        'R': //READY
        begin
                if(TCPServer.Enabled) then
                begin
                        Player2Ready := True;
                        ShapeP2.Show;
                        if(Player1Ready) then
                        begin
                                btnPlay.Enabled := True;
                                StatusBar1.SimpleText := 'Press play to start the game';
                        end;
                end
                else
                begin
                        Player1Ready := True;
                        ShapeP1.Show;
                        if(Player2Ready) then
                        begin
                                btnPlay.Enabled := True;
                                StatusBar1.SimpleText := 'Press play to start the game';
                        end;
                end
        end;
        'N': //NAME
        begin
                //determine player and show name on label
                if(TCPServer.Enabled)then
                begin
                        i := 2;
                        lblPlayer2.Caption := String(Ops[1]);
                end
                else
                begin
                        i := 1;
                        lblPlayer1.Caption := String(Ops[1]);
                end;

                PlayerName[i] := String(Ops[1]); //set name

                //display message
                Application.MessageBox(PChar('You have connnected with ' + PlayerName[i] + '.'),'Notice',MB_OK);

                //set chat window title
                Chat.frmChat.LabelTo.Caption := 'To: ' + PlayerName[i];
        end;
        end; //end case
end;

procedure TFrmBattleShip.DecodeMessage(DataString:String);
var
        Msg : array of Char;
        i,j : Integer;
        L : Integer;
begin
        L := Floor(Length(DataString) + 5);
        if(L >= 250)then
                L := 255;
        SetLength(Msg, L);

        i:=0;
        j:=2;
        while(DataString[j] <> ''#0) and (i < 250) do
         begin
                Msg[i] := Datastring[j];
                i := i+1;
                j := j+1;
         end;

        i:=i-1; //go back to '='
        Msg[i] := ''#0; //overwrite w/null

        PostMessage(String(Msg));
end;

//////////////////////////////////////////////////////////////////
// ENCODING - Encodes and sends to other player
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.EncodeMove(WhosTurn,X,Y:Integer);
begin                                  //=1=X=Y=
        SendData('=MOV='+IntToStr(WhosTurn)+'='+IntToStr(X)+'='+IntToStr(Y)+'=');
end;
procedure TFrmBattleShip.EncodePlace(WhichPlayer:Integer; ShipType : TShip; SX,SY,EX,EY:Integer);
var                                   //=1=2=SX=SY=EX=EY=
        ShipStr : String;
begin
        case ShipType of
        sACarrier: ShipStr := '1';
        sCarrier: ShipStr := '2';
        sBattleship: ShipStr := '3';
        sSubmarine: ShipStr := '4';
        sDestroyer: ShipStr := '5';
        end;

        SendData('=PLC='+IntToStr(WhichPlayer)+'='+ShipStr+'='+IntToStr(SX)+'='+IntToStr(SY)+'='+IntToStr(EX)+'='+IntToStr(EY)+'=');
end;

//////////////////////////////////////////////////////////////////
// InternetBoardMouseDown - Handles all occurances of clicks on
//                          the game board in internet mode.
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.InternetBoardMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Col, Row: Integer;
begin
  if(FreezeGame) then Exit;

  if PlaceProcess then
  begin
    if(TCPServer.Enabled) then //player 1 - SERVER placing
    begin
      if CurrentShipPlaced <> sNone then
      begin
        Board1.MouseToCell(X,Y,Col,Row);
        StatusBar1.SimpleText := PlayerName[1] + ' select the end point for your ' + GetShipName(CurrentShipPlaced) + '.';
        PlaceBoat(1, Col, Row);
        PaintGame();
      end;
    end
    else //player 2 - CLIENT placing
    begin
      if CurrentShipPlaced <> sNone then
      begin
        Board2.MouseToCell(X,Y,Col,Row);
        StatusBar1.SimpleText := PlayerName[2] + ' select the end point for your ' + GetShipName(CurrentShipPlaced) + '.';
        PlaceBoat(2, Col, Row);
        PaintGame();
      end;
    end;
  end;

  //if game in session then handle click.
  if Play then
  begin
    if(TCPServer.Enabled) then
        Board2.MouseToCell(X,Y,Col,Row) //Get Shot coords
    else
        Board1.MouseToCell(X,Y,Col,Row);

    if ((TCPServer.Enabled) and (CurrentPlayer=1)) or ((TCPClient.Enabled) and (CurrentPlayer=2)) then
     begin
        //Player 1 clicks on own board.
        if(Sender = Board1) AND (CurrentPlayer = 1) then
        begin
           //blah
        end
        //Player 2 fires on Player 1.
        else if(Sender = Board1) AND (CurrentPlayer = 2) then
         begin
                CannonFired(1,Col,Row);
                EncodeMove(2,Col,Row); //Send move to opponent
                CheckBoard(1);//Check for sink or win
                EndTurn();
         end
        //Player 1 fires on Player 2.
        else if(Sender = Board2) AND (CurrentPlayer = 1) then
         begin
                CannonFired(2,Col,Row);
                EncodeMove(1,Col,Row); //Send move to opponent
                CheckBoard(2);//Check for sink or win
                EndTurn();
         end
        //Player 2 clicks on own board.
        else if(Sender = Board2) AND (CurrentPlayer = 2) then
         begin
        //blah
         end;
     end
    else // it is not your turn!!!
     begin
        Beep;
        StatusBar1.SimpleText := 'It is not your turn! You must wait until your opponent is finished.';
     end;

    PaintGame(); //Repaint game boards for current player
  end; //end if play 
end;

//////////////////////////////////////////////////////////////////
// InternetbtnDonePlacingClick -
//////////////////////////////////////////////////////////////////
procedure TFrmBattleShip.InternetbtnDonePlacingClick(Sender: TObject);
var
  Status : Boolean;
begin
  if(FreezeGame) then Exit;
  
  Status := CheckStatus();

  if not Status then //not done placing
    StatusBar1.SimpleText := 'You must place ALL your ships.'
  else //done placing
  begin
    if(TCPServer.Enabled) then //Player1 done placeing
    begin
        Player1Ready := True;
        ShapeP1.Show;
        SendData('=OTH=R='); //tell opponent that im ready
        btnDonePlacing.Visible := False;
        btnRandomPlace.Visible := False;

        if not Player2Ready then
                StatusBar1.SimpleText := 'Waiting for '+PlayerName[2]+' to finish placing'
        else
        begin
                btnPlay.Enabled := True;
                StatusBar1.SimpleText := 'Press Play to start the game.';
        end;
    end
    else //player 2 done placing.
    begin
        Player2Ready := True;
        ShapeP2.Show;
        SendData('=OTH=R='); //tell opponent that im ready
        btnDonePlacing.Visible := False;
        btnRandomPlace.Visible := False;

        if not Player1Ready then
                StatusBar1.SimpleText := 'Waiting for '+PlayerName[1]+' to finish placing'
        else
        begin
                btnPlay.Enabled := True;
                StatusBar1.SimpleText := 'Press Play to start the game.';
        end;
    end;

  end;
end;

procedure TFrmBattleShip.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
        if(Key = VK_F8) then
                NewGame1Click(Sender);
end;
 //////////////////////////
procedure TFrmBattleShip.MediaPlayer1Enter(Sender: TObject);
begin
        //MediaPlayer1.Wait := True;
        //MediaPlayer1.open;
        //MediaPlayer1.play;
end;

procedure TFrmBattleShip.MediaPlayer1Notify(Sender: TObject);
begin
        //with MediaPlayer1 do
        //if notifyvalue = nvSuccessful then begin
        //        Position := 0;
       //         Play;
        //end;
end;



//Replay the background music when finished
procedure TFrmBattleShip.MediaPlayerNotify(Sender: TObject);
begin
     with Mediaplayer do
     Position := 0;
     Mediaplayer.Play;
end;

procedure TFrmBattleShip.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
        NS_ACarrierPic.Free;
        NS_CarrierPic.Free;
        NS_BattleshipPic.Free;
        NS_DestroyerPic.Free;
        NS_SubmarinePic.Free;

        WE_ACarrierPic.Free;
        WE_CarrierPic.Free;
        WE_BattleshipPic.Free;
        WE_DestroyerPic.Free;
        WE_SubmarinePic.Free;

        HitPic.Free;
        MissPic.Free;
        PlacementPic.Free;
        CloseFile(Scoresfile);
        writehighScores();
        CloseFile(Scoresfile);
end;



procedure TFrmBattleShip.fillscorearray();
var
             counter : integer;
             score : string;
begin
        counter := 1;
        Assignfile(ScoresFile, 'highscore.txt.txt');
        FileMode := 2;
        reset(ScoresFile);

         while not Eof(ScoresFile) do
         begin
                Readln(ScoresFile, top_scores[counter].name);
                Readln(ScoresFile, score);
                top_scores[counter].score := strtoint(score);
                Readln(ScoresFile, top_scores[counter].date);
                counter := counter + 1;
         end;

end;

procedure TFrmBattleShip.writehighScores();
var
        counter : integer;
        score : string;
begin
        rewrite(ScoresFile);
        for counter := 1 to 10 do
        begin

            score := inttostr(top_scores[counter].score);
            writeln(ScoresFile, top_scores[counter].name);
            writeln(ScoresFile, score);
            writeln(ScoresFile, top_scores[counter].date);
        end;

end;




procedure TFrmBattleship.searchScores(score : integer;WhichPlayer : Integer);

begin
        if(score > top_scores[1].score) then
        begin
                top_scores[1].score := score;
                top_scores[1].name :=  PlayerName[WhichPlayer];
                top_scores[1].date := DateToStr(Date);

        end
        else if(score > top_scores[2].score) then
        begin
                top_scores[2].score := score;
                top_scores[2].name :=  PlayerName[WhichPlayer];
                top_scores[2].date := DateToStr(Date);

        end
        else if(score > top_scores[3].score) then
        begin
                top_scores[3].score := score;
                top_scores[3].name :=  PlayerName[WhichPlayer];
                top_scores[3].date := DateToStr(Date);
        end
        else if(score > top_scores[4].score) then
        begin
                top_scores[4].score := score;
                top_scores[4].name :=  PlayerName[WhichPlayer];
                top_scores[4].date := DateToStr(Date);
        end
        else if(score > top_scores[5].score) then
        begin
                top_scores[5].score := score;
                top_scores[5].name :=  PlayerName[WhichPlayer];
                top_scores[5].date := DateToStr(Date);
        end
        else if(score > top_scores[6].score) then
        begin
                top_scores[6].score := score;
                top_scores[6].name :=  PlayerName[WhichPlayer];
                top_scores[6].date := DateToStr(Date);
        end
        else if(score > top_scores[7].score) then
        begin
                top_scores[7].score := score;
                top_scores[7].name :=  PlayerName[WhichPlayer];
                top_scores[7].date := DateToStr(Date);
        end
        else if(score > top_scores[8].score) then
        begin
                top_scores[8].score := score;
                top_scores[8].name :=  PlayerName[WhichPlayer];
                top_scores[8].date := DateToStr(Date);
        end
        else if(score > top_scores[9].score) then
        begin
                top_scores[9].score := score;
                top_scores[9].name :=  PlayerName[WhichPlayer];
                top_scores[9].date := DateToStr(Date);
        end
        else if(score > top_scores[10].score) then
        begin
                top_scores[10].score := score;
                top_scores[10].name :=  PlayerName[WhichPlayer];
                top_scores[10].date := DateToStr(Date);
        end;

end;






procedure TFrmBattleShip.TopScores1Click(Sender: TObject);
begin
     frm_highscore.showmodal;
end;

end.

